@@ -177,7 +177,8 @@ class DatabaseOverviewMetricGroup(MetricGroupDef):
                          nvcsws, nivcsws])
             from_clause = from_clause.join(
                 kcache_query,
-                kcache_query.c.ts == c.ts)
+                kcache_query.c.ts == c.ts,
+                isouter=True)
 
         return (select(cols)
                 .select_from(from_clause)
@@ -243,7 +244,8 @@ class DatabaseWaitOverviewMetricGroup(MetricGroupDef):
         cols = [to_epoch(c.ts)]
 
         pg_version_num = self.get_pg_version_num(self.path_args[0])
-        if pg_version_num < 100000:
+        # if we can't connect to the remote server, assume pg10 or above
+        if pg_version_num is None or pg_version_num < 100000:
             cols += [wps(c.count_lwlocknamed), wps(c.count_lwlocktranche),
                      wps(c.count_lock), wps(c.count_bufferpin)]
         else:
@@ -556,8 +558,10 @@ class DatabaseOverview(DashboardPage):
             block_graph.color_scheme = ['#cb513a', '#73c03a']
 
         if (self.has_extension(self.path_args[0], "pg_wait_sampling")):
-            metrics=None
-            if self.get_pg_version_num(self.path_args[0]) < 100000:
+            metrics = None
+            pg_version_num = self.get_pg_version_num(self.path_args[0])
+            # if we can't connect to the remote server, assume pg10 or above
+            if pg_version_num is None or pg_version_num < 100000:
                 metrics = [DatabaseWaitOverviewMetricGroup.count_lwlocknamed,
                            DatabaseWaitOverviewMetricGroup.count_lwlocktranche,
                            DatabaseWaitOverviewMetricGroup.count_lock,
