@@ -252,7 +252,8 @@ class QueryOverviewMetricGroup(MetricGroupDef):
                 and_(kcache_query.c.ts == c.ts,
                      kcache_query.c.queryid == c.queryid,
                      kcache_query.c.userid == c.userid,
-                     kcache_query.c.dbid == c.dbid))
+                     kcache_query.c.dbid == c.dbid),
+                isouter=True)
         else:
             cols.extend([
                 case([(total_blocks == 0, 0)],
@@ -445,7 +446,8 @@ class WaitsQueryOverviewMetricGroup(MetricGroupDef):
         cols = [to_epoch(c.ts)]
 
         pg_version_num = self.get_pg_version_num(self.path_args[0])
-        if pg_version_num < 100000:
+        # if we can't connect to the remote server, assume pg10 or above
+        if pg_version_num is None or pg_version_num < 100000:
             cols += [wps(c.count_lwlocknamed), wps(c.count_lwlocktranche),
                      wps(c.count_lock), wps(c.count_bufferpin)]
         else:
@@ -567,7 +569,9 @@ class QueryDetail(ContentWidget):
             wblk,
             (rblk + wblk).label("total_blks")])
             .select_from(from_clause)
-            .where(powa_statements.c.queryid == bindparam("query"))
+            .where(
+                (powa_statements.c.queryid == bindparam("query")) &
+                (powa_statements.c.srvid == bindparam("server")))
             .group_by(column("query"), bs))
 
         value = self.execute(stmt, params={
@@ -690,8 +694,10 @@ class QueryOverview(DashboardPage):
 
         if self.has_extension(self.path_args[0], "pg_wait_sampling"):
             # Get the metrics depending on the pg server version
-            metrics=None
-            if self.get_pg_version_num(self.path_args[0]) < 100000:
+            metrics = None
+            pg_version_num = self.get_pg_version_num(self.path_args[0])
+            # if we can't connect to the remote server, assume pg10 or above
+            if pg_version_num is None or pg_version_num < 100000:
                 metrics=[WaitsQueryOverviewMetricGroup.count_lwlocknamed,
                          WaitsQueryOverviewMetricGroup.count_lwlocktranche,
                          WaitsQueryOverviewMetricGroup.count_lock,
