@@ -39,15 +39,92 @@ class Biggestsum(object):
 
 
 def powa_base_statdata_detailed_db():
+    """
+    Base for query used in the grids displaying info about pgss.
+
+    This is based on the "detailed" version of the table, with queryid
+    information.
+
+    As the data is stored in 2 sets of tables (the coalesced records and the
+    "current" records), we have to retrieve the query in multiple steps.
+
+    To improve performance, the retrieval of the coalesced records is divided
+    in 3 parts, which can greatly limit the number of rows that will need to be
+    sorted.
+
+    Those 3 parts are split in 3 subqueries.  For the left and right bounds
+    (the smallest and largest timestamp in the given interval), we get the 2
+    underlying records in the coalesced records (if any) and unnest them
+    fully, filtering any records outside of the interval bound.  For any record
+    in the coalesced records that are entirely inside the given interval,
+    we rely on the pre-computed metadata (mins_in_range and maxs_in_range) and
+    simply return that, which can greatly reduces the amount of rows to sort.
+
+    For the "current" records, we simply return all the rows in the given
+    interval.
+    """
     base_query = text("""
   powa_databases,
   LATERAL
   (
-    SELECT unnested.dbid, unnested.userid, unnested.queryid,
+    -- Left bound: the search interval is a single timestamp, the smallest one
+    -- of the search interval, and has to be inside the coalesce_range. We
+    -- still need to unnest this one as we may have to remove some of the
+    -- underlying records
+    SELECT unnested.dbid, unnested.toplevel, unnested.userid, unnested.queryid,
       (unnested.records).*
     FROM (
-      SELECT psh.dbid, psh.userid, psh.queryid, psh.coalesce_range,
-        unnest(records) AS records
+      SELECT psh.dbid, psh.toplevel, psh.userid, psh.queryid,
+        psh.coalesce_range, unnest(records) AS records
+      FROM powa_statements_history psh
+      WHERE coalesce_range && tstzrange(:from, :from, '[]')
+      AND psh.dbid = powa_databases.oid
+      AND psh.queryid IN (
+        SELECT powa_statements.queryid
+        FROM powa_statements
+        WHERE powa_statements.dbid = powa_databases.oid
+          AND powa_statements.srvid = :server
+      )
+      AND psh.srvid = :server
+    ) AS unnested
+    WHERE (records).ts <@ tstzrange(:from, :to, '[]')
+
+    UNION ALL
+
+    -- Right bound: the search interval is a single timestamp, the largest one
+    -- of the search interval, and has to be inside the coalesce_range. We
+    -- still need to unnest this one as we may have to remove some of the
+    -- underlying records
+    SELECT unnested.dbid, unnested.toplevel, unnested.userid, unnested.queryid,
+      (unnested.records).*
+    FROM (
+      SELECT psh.dbid, psh.toplevel, psh.userid, psh.queryid,
+        psh.coalesce_range, unnest(records) AS records
+      FROM powa_statements_history psh
+      WHERE coalesce_range && tstzrange(:to, :to, '[]')
+      AND psh.dbid = powa_databases.oid
+      AND psh.queryid IN (
+        SELECT powa_statements.queryid
+        FROM powa_statements
+        WHERE powa_statements.dbid = powa_databases.oid
+          AND powa_statements.srvid = :server
+      )
+      AND psh.srvid = :server
+    ) AS unnested
+    WHERE (records).ts <@ tstzrange(:from, :to, '[]')
+
+    UNION ALL
+
+    -- These entries have their coalesce_range ENTIRELY inside the search range
+    -- so we don't need to unnest them.  We just retrieve the mins_in_range,
+    -- maxs_in_range from the record, build an array of this and return it as
+    -- if it was the full record
+    SELECT unnested.dbid, unnested.toplevel, unnested.userid, unnested.queryid,
+      (unnested.records).*
+    FROM (
+      SELECT psh.dbid, psh.toplevel, psh.userid, psh.queryid,
+        psh.coalesce_range,
+        unnest(ARRAY[mins_in_range,maxs_in_range]) AS records
       FROM powa_statements_history psh
       WHERE coalesce_range && tstzrange(:from, :to, '[]')
       AND psh.dbid = powa_databases.oid
@@ -60,8 +137,11 @@ def powa_base_statdata_detailed_db():
       AND psh.srvid = :server
     ) AS unnested
     WHERE (records).ts <@ tstzrange(:from, :to, '[]')
+
     UNION ALL
-    SELECT psc.dbid, psc.userid, psc.queryid,(psc.record).*
+
+    -- The "current" records are simply returned after filtering
+    SELECT psc.dbid, psc.toplevel, psc.userid, psc.queryid,(psc.record).*
     FROM powa_statements_history_current psc
     WHERE (record).ts <@ tstzrange(:from,:to,'[]')
     AND psc.dbid = powa_databases.oid
@@ -77,6 +157,14 @@ def powa_base_statdata_detailed_db():
 
 
 def powa_base_statdata_db():
+    """
+    Query used in the grids displaying info about pgss.
+
+    This is based on the db-aggregated version of the tables, without queryid
+    information.
+
+    This uses the same optimization as powa_base_statdata_detailed_db.
+    """
     base_query = text("""(
  SELECT d.srvid, d.oid as dbid, h.*
  FROM
@@ -89,10 +177,46 @@ def powa_base_statdata_db():
    GROUP BY srvid, dbid
  ) ranges ON d.oid = ranges.dbid AND d.srvid = ranges.srvid,
  LATERAL (
+   -- Left bound: the search interval is a single timestamp, the smallest one
+   -- of the search interval, and has to be inside the coalesce_range. We
+   -- still need to unnest this one as we may have to remove some of the
+   -- underlying records
    SELECT (unnested1.records).*
    FROM (
      SELECT dbh.coalesce_range, unnest(records) AS records
      FROM powa_statements_history_db dbh
+     WHERE coalesce_range && tstzrange(:from, :from, '[]')
+     AND dbh.dbid = ranges.dbid
+     AND dbh.srvid = :server
+   ) AS unnested1
+   WHERE (unnested1.records).ts <@ tstzrange(:from, :to, '[]')
+
+   -- Right bound: the search interval is a single timestamp, the largest one
+   -- of the search interval, and has to be inside the coalesce_range. We
+   -- still need to unnest this one as we may have to remove some of the
+   -- underlying records
+   UNION ALL
+   SELECT (unnested2.records).*
+   FROM (
+     SELECT dbh.coalesce_range, unnest(records) AS records
+     FROM powa_statements_history_db dbh
+     WHERE coalesce_range && tstzrange(:to, :to, '[]')
+     AND dbh.dbid = ranges.dbid
+     AND dbh.srvid = :server
+   ) AS unnested2
+   WHERE  (unnested2.records).ts <@ tstzrange(:from, :to, '[]')
+
+   UNION ALL
+
+   -- These entries have their coalesce_range ENTIRELY inside the search range
+   -- so we don't need to unnest them.  We just retrieve the mins_in_range,
+   -- maxs_in_range from the record, build an array of this and return it as
+   -- if it was the full record
+   SELECT (unnested1.records).*
+   FROM (
+     SELECT dbh.coalesce_range,
+       unnest(ARRAY[mins_in_range,maxs_in_range]) AS records
+     FROM powa_statements_history_db dbh
      WHERE coalesce_range && tstzrange(:from, :to, '[]')
      AND dbh.dbid = ranges.dbid
      AND dbh.srvid = :server
@@ -108,7 +232,10 @@ def powa_base_statdata_db():
      AND dbh.srvid = :server
    ) AS unnested2
    WHERE  (unnested2.records).ts <@ tstzrange(:from, :to, '[]')
+
    UNION ALL
+
+   -- The "current" records are simply returned after filtering
    SELECT (dbc.record).*
    FROM powa_statements_history_current_db dbc
    WHERE  (dbc.record).ts <@ tstzrange(:from, :to, '[]')
@@ -156,7 +283,8 @@ def powa_base_bgwriter():
 def get_diffs_forstatdata():
     return [
         diff("calls"),
-        diff("total_time").label("runtime"),
+        diff("total_plan_time").label("plantime"),
+        diff("total_exec_time").label("runtime"),
         diff("shared_blks_read"),
         diff("shared_blks_hit"),
         diff("shared_blks_dirtied"),
@@ -164,28 +292,44 @@ def get_diffs_forstatdata():
         diff("temp_blks_read"),
         diff("temp_blks_written"),
         diff("blk_read_time"),
-        diff("blk_write_time")
+        diff("blk_write_time"),
+        diff("wal_records"),
+        diff("wal_fpi"),
+        diff("wal_bytes")
     ]
 
 
 def powa_getstatdata_detailed_db(srvid):
+    """
+    Query used in the grids displaying info about pgss.
+
+    This is based on the "detailed" version of the tables, with queryid
+    information.
+    """
     base_query = powa_base_statdata_detailed_db()
     diffs = get_diffs_forstatdata()
     return (select([
         column("srvid"),
         column("queryid"),
         column("dbid"),
+        column("toplevel"),
         column("userid"),
         column("datname"),
     ] + diffs)
             .select_from(base_query)
             .where(column("srvid") == srvid)
             .group_by(column("srvid"), column("queryid"), column("dbid"),
-                      column("userid"), column("datname"))
+                      column("toplevel"), column("userid"), column("datname"))
             .having(max(column("calls")) - min(column("calls")) > 0))
 
 
 def powa_getstatdata_db(srvid):
+    """
+    Query used in the grids displaying info about pgss.
+
+    This is based on the db-aggregated version of the tables, without queryid
+    information.
+    """
     base_query = powa_base_statdata_db()
     diffs = get_diffs_forstatdata()
     return (select([column("srvid")] + [column("dbid")] + diffs)
@@ -292,7 +436,8 @@ def powa_getstatdata_sample(mode, srvid):
         ts,
         biggest("ts", '0 s', "mesure_interval"),
         biggestsum("calls"),
-        biggestsum("total_time", label="runtime"),
+        biggestsum("total_plan_time").label("plantime"),
+        biggestsum("total_exec_time", label="runtime"),
         biggestsum("rows"),
         biggestsum("shared_blks_read"),
         biggestsum("shared_blks_hit"),
@@ -305,7 +450,11 @@ def powa_getstatdata_sample(mode, srvid):
         biggestsum("temp_blks_read"),
         biggestsum("temp_blks_written"),
         biggestsum("blk_read_time"),
-        biggestsum("blk_write_time")])
+        biggestsum("blk_write_time"),
+        biggestsum("wal_records"),
+        biggestsum("wal_fpi"),
+        biggestsum("wal_bytes")
+        ])
             .select_from(base_query)
             .apply_labels()
             .group_by(*(base_columns + [ts])))
@@ -426,18 +575,19 @@ BASE_QUERY_KCACHE_SAMPLE_DB = text("""
                         *
                 FROM (
                     SELECT km.ts,
-                    sum(km.reads) AS reads, sum(km.writes) AS writes,
-                    sum(km.user_time) AS user_time,
-                    sum(km.system_time) AS system_time,
-                    sum(km.minflts) AS minflts,
-                    sum(km.majflts) AS majflts,
+                    sum(km.plan_reads + km.exec_reads) AS reads,
+                    sum(km.plan_writes + km.exec_writes) AS writes,
+                    sum(km.plan_user_time + km.exec_user_time) AS user_time,
+                    sum(km.plan_system_time + km.exec_system_time) AS system_time,
+                    sum(km.plan_minflts + km.exec_minflts) AS minflts,
+                    sum(km.plan_majflts + km.exec_majflts) AS majflts,
                     -- not maintained on GNU/Linux, and not available on Windows
-                    -- sum(km.nswaps) AS nswaps,
-                    -- sum(km.msgsnds) AS msgsnds,
-                    -- sum(km.msgrcvs) AS msgrcvs,
-                    -- sum(km.nsignals) AS nsignals,
-                    sum(km.nvcsws) AS nvcsws,
-                    sum(km.nivcsws) AS nivcsws
+                    -- sum(km.plan_nswaps + km.exec_nswaps) AS nswaps,
+                    -- sum(km.plan_msgsnds + km.exec_msgsnds) AS msgsnds,
+                    -- sum(km.plan_msgrcvs + km.exec_msgrcvs) AS msgrcvs,
+                    -- sum(km.plan_nsignals + km.exec_nsignals) AS nsignals,
+                    sum(km.plan_nvcsws + km.exec_nvcsws) AS nvcsws,
+                    sum(km.plan_nivcsws + km.exec_nivcsws) AS nivcsws
                     FROM (
                         SELECT * FROM (
                             SELECT (unnest(metrics)).*
@@ -475,18 +625,19 @@ BASE_QUERY_KCACHE_SAMPLE = text("""
                         *
                 FROM (
                     SELECT km.ts,
-                    sum(km.reads) AS reads, sum(km.writes) AS writes,
-                    sum(km.user_time) AS user_time,
-                    sum(km.system_time) AS system_time,
-                    sum(km.minflts) AS minflts,
-                    sum(km.majflts) AS majflts,
+                    sum(km.plan_reads + km.exec_reads) AS reads,
+                    sum(km.plan_writes + km.exec_writes) AS writes,
+                    sum(km.plan_user_time + km.exec_user_time) AS user_time,
+                    sum(km.plan_system_time + km.exec_system_time) AS system_time,
+                    sum(km.plan_minflts + km.exec_minflts) AS minflts,
+                    sum(km.plan_majflts + km.exec_majflts) AS majflts,
                     -- not maintained on GNU/Linux, and not available on Windows
-                    -- sum(km.nswaps) AS nswaps,
-                    -- sum(km.msgsnds) AS msgsnds,
-                    -- sum(km.msgrcvs) AS msgrcvs,
-                    -- sum(km.nsignals) AS nsignals,
-                    sum(km.nvcsws) AS nvcsws,
-                    sum(km.nivcsws) AS nivcsws
+                    -- sum(km.plan_nswaps + km.exec_nswaps) AS nswaps,
+                    -- sum(km.plan_msgsnds + km.exec_msgsnds) AS msgsnds,
+                    -- sum(km.plan_msgrcvs + km.exec_msgrcvs) AS msgrcvs,
+                    -- sum(km.plan_nsignals + km.exec_nsignals) AS nsignals,
+                    sum(km.plan_nvcsws + km.exec_nvcsws) AS nvcsws,
+                    sum(km.plan_nivcsws + km.exec_nivcsws) AS nivcsws
                     FROM (
                         SELECT * FROM (
                             SELECT (unnest(metrics)).*
@@ -706,18 +857,32 @@ BASE_QUERY_BGWRITER_SAMPLE = text("""
 
 
 def powa_base_waitdata_detailed_db():
+    """
+    Query used in the grids displaying info about pg_wait_sampling.
+
+    This is based on the "detailed" version of the tables, with queryid
+    information.
+
+    This uses the same optimization as powa_base_statdata_detailed_db.
+    """
     base_query = text("""
   powa_databases,
   LATERAL
   (
+    -- Left bound: the search interval is a single timestamp, the smallest one
+    -- of the search interval, and has to be inside the coalesce_range. We
+    -- still need to unnest this one as we may have to remove some of the
+    -- underlying records
     SELECT unnested.dbid, unnested.queryid,
       unnested.event_type, unnested.event, (unnested.records).*
     FROM (
       SELECT wsh.dbid, wsh.queryid, wsh.event_type, wsh.event,
         wsh.coalesce_range, unnest(records) AS records
       FROM powa_wait_sampling_history wsh
-      WHERE coalesce_range && tstzrange(:from, :to, '[]')
+      WHERE coalesce_range && tstzrange(:from, :from, '[]')
       AND wsh.dbid = powa_databases.oid
+      -- we can't simply join powa_statements as there's no userid in
+      -- powa_wait_sampling_* tables
       AND wsh.queryid IN (
         SELECT ps.queryid
         FROM powa_statements ps
@@ -727,11 +892,69 @@ def powa_base_waitdata_detailed_db():
       AND wsh.srvid = :server
     ) AS unnested
     WHERE  (records).ts <@ tstzrange(:from, :to, '[]')
+
     UNION ALL
+
+    -- Right bound: the search interval is a single timestamp, the largest one
+    -- of the search interval, and has to be inside the coalesce_range. We
+    -- still need to unnest this one as we may have to remove some of the
+    -- underlying records
+    SELECT unnested.dbid, unnested.queryid,
+      unnested.event_type, unnested.event, (unnested.records).*
+    FROM (
+      SELECT wsh.dbid, wsh.queryid, wsh.event_type, wsh.event,
+        wsh.coalesce_range, unnest(records) AS records
+      FROM powa_wait_sampling_history wsh
+      WHERE coalesce_range && tstzrange(:to, :to, '[]')
+      AND wsh.dbid = powa_databases.oid
+      -- we can't simply join powa_statements as there's no userid in
+      -- powa_wait_sampling_* tables
+      AND wsh.queryid IN (
+        SELECT ps.queryid
+        FROM powa_statements ps
+        WHERE ps.dbid = powa_databases.oid
+          AND ps.srvid = :server
+      )
+      AND wsh.srvid = :server
+    ) AS unnested
+    WHERE  (records).ts <@ tstzrange(:from, :to, '[]')
+
+    UNION ALL
+
+    -- These entries have their coalesce_range ENTIRELY inside the search range
+    -- so we don't need to unnest them.  We just retrieve the mins_in_range,
+    -- maxs_in_range from the record, build an array of this and return it as
+    -- if it was the full record
+    SELECT unnested.dbid, unnested.queryid,
+      unnested.event_type, unnested.event, (unnested.records).*
+    FROM (
+      SELECT wsh.dbid, wsh.queryid, wsh.event_type, wsh.event,
+        wsh.coalesce_range,
+        unnest(ARRAY[mins_in_range,maxs_in_range]) AS records
+      FROM powa_wait_sampling_history wsh
+      WHERE coalesce_range && tstzrange(:from, :to, '[]')
+      AND wsh.dbid = powa_databases.oid
+      -- we can't simply join powa_statements as there's no userid in
+      -- powa_wait_sampling_* tables
+      AND wsh.queryid IN (
+        SELECT ps.queryid
+        FROM powa_statements ps
+        WHERE ps.dbid = powa_databases.oid
+          AND ps.srvid = :server
+      )
+      AND wsh.srvid = :server
+    ) AS unnested
+    WHERE  (records).ts <@ tstzrange(:from, :to, '[]')
+
+    UNION ALL
+
+    -- The "current" records are simply returned after filtering
     SELECT wsc.dbid, wsc.queryid, wsc.event_type, wsc.event, (wsc.record).*
     FROM powa_wait_sampling_history_current wsc
     WHERE (record).ts <@ tstzrange(:from,:to,'[]')
     AND wsc.dbid = powa_databases.oid
+    -- we can't simply join powa_statements as there's no userid in
+    -- powa_wait_sampling_* tables
     AND wsc.queryid IN (
       SELECT ps.queryid
       FROM powa_statements ps
@@ -746,6 +969,14 @@ def powa_base_waitdata_detailed_db():
 
 
 def powa_base_waitdata_db():
+    """
+    Query used in the grids displaying info about pg_wait_sampling.
+
+    This is based on the db-aggregated version of the tables, without queryid
+    information.
+
+    This uses the same optimization as powa_base_statdata_detailed_db.
+    """
     base_query = text("""(
   SELECT powa_databases.srvid, powa_databases.oid as dbid, h.*
   FROM
@@ -758,16 +989,56 @@ def powa_base_waitdata_db():
     GROUP BY dbid
   ) ranges ON powa_databases.oid = ranges.dbid,
   LATERAL (
+    -- Left bound: the search interval is a single timestamp, the smallest one
+    -- of the search interval, and has to be inside the coalesce_range. We
+    -- still need to unnest this one as we may have to remove some of the
+    -- underlying records
     SELECT event_type, event, (unnested1.records).*
     FROM (
       SELECT wsh.event_type, wsh.event, unnest(records) AS records
       FROM powa_wait_sampling_history_db wsh
+      WHERE coalesce_range && tstzrange(:from, :from, '[]')
+      AND wsh.dbid = ranges.dbid
+      AND wsh.srvid = :server
+    ) AS unnested1
+    WHERE (unnested1.records).ts <@ tstzrange(:from, :to, '[]')
+
+    UNION ALL
+
+    -- Right bound: the search interval is a single timestamp, the largest one
+    -- of the search interval, and has to be inside the coalesce_range. We
+    -- still need to unnest this one as we may have to remove some of the
+    -- underlying records
+    SELECT event_type, event, (unnested1.records).*
+    FROM (
+      SELECT wsh.event_type, wsh.event, unnest(records) AS records
+      FROM powa_wait_sampling_history_db wsh
+      WHERE coalesce_range && tstzrange(:to, :to, '[]')
+      AND wsh.dbid = ranges.dbid
+      AND wsh.srvid = :server
+    ) AS unnested1
+    WHERE (unnested1.records).ts <@ tstzrange(:from, :to, '[]')
+
+    UNION ALL
+
+    -- These entries have their coalesce_range ENTIRELY inside the search range
+    -- so we don't need to unnest them.  We just retrieve the mins_in_range,
+    -- maxs_in_range from the record, build an array of this and return it as
+    -- if it was the full record
+    SELECT event_type, event, (unnested1.records).*
+    FROM (
+      SELECT wsh.event_type, wsh.event,
+        unnest(ARRAY[mins_in_range,maxs_in_range]) AS records
+      FROM powa_wait_sampling_history_db wsh
       WHERE coalesce_range && tstzrange(:from, :to, '[]')
       AND wsh.dbid = ranges.dbid
       AND wsh.srvid = :server
     ) AS unnested1
     WHERE (unnested1.records).ts <@ tstzrange(:from, :to, '[]')
+
     UNION ALL
+
+    -- The "current" records are simply returned after filtering
     SELECT event_type, event, (wsc.record).*
     FROM powa_wait_sampling_history_current_db wsc
     WHERE (wsc.record).ts <@ tstzrange(:from, :to, '[]')
@@ -873,6 +1144,12 @@ BASE_QUERY_ALL_RELS_SAMPLE = text("""(
 
 
 def powa_getwaitdata_detailed_db(srvid):
+    """
+    Query used in the grids displaying info about pg_wait_sampling.
+
+    This is based on the "detailed" version of the tables, with queryid
+    information.
+    """
     base_query = powa_base_waitdata_detailed_db()
     return (select([
         column("srvid"),
@@ -890,6 +1167,12 @@ def powa_getwaitdata_detailed_db(srvid):
 
 
 def powa_getwaitdata_db(srvid):
+    """
+    Query used in the grids displaying info about pg_wait_sampling.
+
+    This is based on the db-aggregated version of the tables, without queryid
+    information.
+    """
     base_query = powa_base_waitdata_db()
 
     return (select([
